package org.netkernel.layer1.endpoint;

// Author: Tom Geudens
// Date  : 2013/03/07

// The usual suspects for an accessor.
import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.meta.impl.SourcedArgumentMetaImpl;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

// Processing.
import org.netkernel.layer0.representation.impl.HDSBuilder;


public class GoldenThreadListAccessor extends StandardAccessorImpl {

	public GoldenThreadListAccessor() {
		this.declareThreadSafe();
		this.declareArgument(new SourcedArgumentMetaImpl("filter",null,null,new Class[] {String.class}));
		this.declareArgument(new SourcedArgumentMetaImpl("offset",null,null,new Class[] {Integer.class}));
		this.declareArgument(new SourcedArgumentMetaImpl("limit",null,null,new Class[] {Integer.class}));
	}

	public void onSource(INKFRequestContext aContext) throws Exception {

		GoldenThreadAccessor gta=GoldenThreadAccessor.getSingleton();
		
		// SOURCE has three optional arguments, filter, offset and limit.
		
		// filter
		String aFilter = null;
		if (aContext.getThisRequest().argumentExists("filter")) {
			aFilter = aContext.source("arg:filter", String.class);
			if (aFilter.equals("")) {
				aFilter = null;
			}
		}
		//
		
		// offset
		Integer aOffset = null;
		if (aContext.getThisRequest().argumentExists("offset")) {
			aOffset = aContext.source("arg:offset", Integer.class);
		}
		else {
			aOffset = 0;
		}	
		//

		// limit
		Integer aLimit = null;
		if (aContext.getThisRequest().argumentExists("limit")) {
			aLimit = aContext.source("arg:limit", Integer.class);
		}
		//
		
		INKFExpiryFunction expiry=gta.getGroupDirtyExpiry();
		
		HDSBuilder vResultBuilder = new HDSBuilder();
		vResultBuilder.pushNode("goldenthreads");

		Integer vItemCount = 0;
		Integer vSelectCount = 0;
		
		for (Object vId : gta.getGroup().listIds()) {
			
			String vIdString = (String)vId;
			
			Boolean vMatch = false;
			
			if (aFilter == null) {
				vMatch = true;
			}
			else {
				for(String vTerm : aFilter.split(" ")) {
					if (vIdString.contains(vTerm)) {
						vMatch = true;
						break;
					}
				}
			}
			
			if (vMatch) {
				if ((vItemCount >= aOffset) && (aLimit==null || (vSelectCount < aLimit) ) ) {
					vResultBuilder.addNode("id", vIdString);
					vSelectCount = vSelectCount + 1;
				}
				vItemCount = vItemCount + 1;
			}
		}
		vResultBuilder.addNode("totalCount", vItemCount);
		
		vResultBuilder.popNode();
		
		aContext.createResponseFrom(vResultBuilder.getRoot()).setExpiry(INKFResponse.EXPIRY_FUNCTION, expiry);
	}
}
