package org.netkernel.layer1.representation;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.netkernel.layer0.representation.ILongReadableBinaryStreamRepresentation;
import org.netkernel.layer0.util.Utils;

public class FileRepresentation implements ILongReadableBinaryStreamRepresentation
{	private final File mFile;
	
	public FileRepresentation(File aFile) throws IOException
	{	mFile = aFile;
	}
	
	/** return the length of the resource */
	public int getContentLength()
	{	if(mFile.length()>(long)Integer.MAX_VALUE)
		{	throw new IllegalStateException("File size too large for integer value  - use getLongContentLength()");
		}
		return (int)mFile.length();
	}
	
	/** return the long length of the file resource */
	public long getLongContentLength()
	{	return mFile.length();
	}
	
	/** write the resource to the given output stream */
	public  void write(OutputStream aStream) throws IOException
	{	InputStream is = new FileInputStream(mFile);
		Utils.pipe(is,aStream);
	}
	
	/** get an input stream for the  resource */
	public synchronized InputStream getInputStream() throws IOException
	{	return new FileInputStream(mFile);
	}
	
	public String getEncoding()
	{	return null;
	}
	
	protected File getFileAdvanced()
	{	return mFile;		
	}
	
}