package org.netkernel.layer1.util;

import org.netkernel.layer0.nkf.INKFExpiryFunction;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

public class GoldenThreadGroup
{
	private Map<Object,GoldenThread> mThreadIdToExpiry;
	
	public GoldenThreadGroup()
	{	mThreadIdToExpiry = new ConcurrentHashMap<Object,GoldenThread>();
	}
	
	public int size()
	{	return mThreadIdToExpiry.size();
	}
	
	public void cut(Object aThreadId)
	{	GoldenThread removed=mThreadIdToExpiry.remove(aThreadId);
		if (removed!=null)
		{	removed.cut();
		}
	}
	
	public INKFExpiryFunction attach(Object aThreadId)
	{	GoldenThread result = mThreadIdToExpiry.get(aThreadId);
		if (result==null)
		{	result = new GoldenThread(aThreadId);
			GoldenThread existing = mThreadIdToExpiry.put(aThreadId, result);
			if (existing!=null)
			{	// someone beat us to it
				mThreadIdToExpiry.put(aThreadId, existing);
				result=existing;
			}
		}
		return result;
	}

	public Set<Object> listIds() {
		return mThreadIdToExpiry.keySet();
	}

	public GoldenThread exists(Object aThreadId)
	{	GoldenThread result = mThreadIdToExpiry.get(aThreadId);
		if(result!=null  && result.isExpired(0))
		{	result=null;			
		}
		return result;
	}
	
	private static class GoldenThread implements INKFExpiryFunction
	{
		private Object mId;
		private boolean mCut;
		
		public GoldenThread(Object aId)
		{	mId=aId;
		}
		
		public void cut()
		{	mCut=true;
		}
		
		public boolean isExpired(long aNow)
		{	return mCut;
		}
		
		public String toString()
		{	return "GoldenThread("+mId.toString()+")";
		}
	}
}
