package org.netkernel.layer1.util;

import org.netkernel.layer0.nkf.INKFExpiryFunction;

import java.lang.ref.ReferenceQueue;
import java.lang.ref.WeakReference;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

/* drop in replacement for GoldenThreadGroup which will allow GC of unreferenced golden threads */
public class GoldenThreadGroup2
{
	private ConcurrentHashMap<Object,WeakValueRef> mThreadIdToExpiry;
	private ReferenceQueue<GoldenThread> mReferenceQueue=new ReferenceQueue<>();
	
	public GoldenThreadGroup2()
	{	mThreadIdToExpiry = new ConcurrentHashMap<Object,WeakValueRef>();
	}
	
	public int size()
	{	return mThreadIdToExpiry.size();
	}
	
	private void processReferenceQueue()
	{	WeakValueRef ref;
		while((ref=(WeakValueRef)mReferenceQueue.poll())!=null)
		{	mThreadIdToExpiry.remove(ref.getKey(),ref);
		}
	}
	
	public boolean cut(Object aThreadId)
	{	processReferenceQueue();
		boolean result=false;
		WeakValueRef removedReference=mThreadIdToExpiry.remove(aThreadId);
		if (removedReference!=null)
		{	GoldenThread removed=removedReference.getValue();
			if (removed!=null)
			{	removed.cut();
				result=true;
			}
		}
		return result;
	}
	
	public INKFExpiryFunction attach(Object aThreadId)
	{	processReferenceQueue();
		GoldenThread result=null;
		WeakValueRef reference=mThreadIdToExpiry.get(aThreadId);
		if (reference!=null)
		{	result=reference.getValue();
			//remove GC'd references
			if (result==null)
			{	mThreadIdToExpiry.remove(aThreadId, reference);
			}
		}
		if (result==null)
		{	result = new GoldenThread(aThreadId);
			WeakValueRef newReference = new WeakValueRef(aThreadId, result, mReferenceQueue);
			reference = mThreadIdToExpiry.putIfAbsent(aThreadId, newReference);
			if (reference!=null)
			{	result=reference.getValue();
				if (newReference!=reference)
				{	//clear this unused instance now so it doesn't get enqueued
					newReference.clear();
				}
			}
		}
		return result;
	}

	public Set<Object> listIds() {
		return mThreadIdToExpiry.keySet();
	}

	public GoldenThread exists(Object aThreadId)
	{	GoldenThread result=null;
		WeakValueRef reference=mThreadIdToExpiry.get(aThreadId);
		if (reference!=null)
		{	GoldenThread thread=reference.getValue();
			if (thread!=null)
			{	if (!thread.isExpired(0))
				{	result=thread;
				}
			}
		}
		return result;
	}
	
	private static class GoldenThread implements INKFExpiryFunction
	{
		private Object mId;
		private boolean mCut;
		
		public GoldenThread(Object aId)
		{	mId=aId;
		}
		
		public void cut()
		{	mCut=true;
		}
		
		public boolean isExpired(long aNow)
		{	return mCut;
		}
		
		public String toString()
		{	return "GoldenThread("+mId.toString()+")";
		}
	}
	
	private static class WeakValueRef extends WeakReference
	{
		private final Object mKey;

		private WeakValueRef(Object aKey, GoldenThread aValue, ReferenceQueue aQueue)
		{	super(aValue, aQueue);
			mKey = aKey;
		}

		public Object getKey()
		{	return mKey;
		}
		
		public GoldenThread getValue()
		{	return (GoldenThread)get();
		}

	}
}
